﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Security.Claims;
using System.Threading.Tasks;
using System.Web;
using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.EntityFramework;
using Microsoft.AspNet.Identity.Owin;
using Microsoft.Owin;
using Microsoft.Owin.Security;
using THEasy.Models;
using Techorizon.Authorization.Common;
using THEasy.Model;
using System.Globalization;
using THEasy.Authentication;

namespace THEasy
{
    // Configure the application user manager used in this application. UserManager is defined in ASP.NET Identity and is used by the application.
    public class ApplicationUserManager : UserManager<User, int>
    {
        public ApplicationUserManager(IUserStore<User, int> store) : base(store)
        {
        }

        public override async Task<IdentityResult> CreateAsync(User user, string password)
        {
            if (user.IsAdUser)
            {
                await Store.CreateAsync(user);
                return IdentityResult.Success;
            }
            else
            {
                return await base.CreateAsync(user, password);
            }
        }


        public override async Task<IdentityResult> ResetPasswordAsync(int userId, string token, string newPassword)
        {
            var user = await Store.FindByIdAsync(userId);
            if (user == null)
            {
                throw new InvalidOperationException(String.Format("Invalid User: '{0}'", userId));
            }

            var passwordStore = Store as IUserPasswordStore<User, int>;
            var result = await PasswordValidator.ValidateAsync(newPassword);
            if (!result.Succeeded)
            {
                return result;
            }
            if (passwordStore != null)
                await passwordStore.SetPasswordHashAsync(user, PasswordHasher.HashPassword(newPassword));

            return await UpdateAsync(user);
            //return await  base.ResetPasswordAsync(userId, token, newPassword);
        }

        public override async Task<ClaimsIdentity> CreateIdentityAsync(User user, string authenticationType)
        {
            var identity = await base.CreateIdentityAsync(user, authenticationType);
            identity.AddClaim(new Claim("LastName", user.LastName ?? ""));
            identity.AddClaim(new Claim("FirstName", user.FirstName ?? ""));
            identity.AddClaim(new Claim("Email", user.Email ?? ""));
            identity.AddClaim(new Claim("IsAdUser", user.IsAdUser.ToString(CultureInfo.InvariantCulture)));

            //TODO: ACL 
            //await user.BuildACLAsync();
            //identity.AddClaim(new Claim("ACL", JsonConvert.SerializeObject(user.Acls)));
            return identity;
        }

        public static ApplicationUserManager Create(IdentityFactoryOptions<ApplicationUserManager> options, IOwinContext context)
        {
            var manager = new ApplicationUserManager(new UserStore());
            // Configure validation logic for usernames
            manager.UserValidator = new UserValidator<User, int>(manager)
            {
                AllowOnlyAlphanumericUserNames = false,
                RequireUniqueEmail = true,

            };

            // Configure validation logic for passwords
            manager.PasswordValidator = new PasswordValidator
            {
                RequiredLength = 8,
                //RequireNonLetterOrDigit = true,
                RequireDigit = true,
                RequireLowercase = true,
                //RequireUppercase = true,
            };

            // Configure user lockout defaults
            manager.UserLockoutEnabledByDefault = true;
            manager.DefaultAccountLockoutTimeSpan = TimeSpan.FromMinutes(5);
            manager.MaxFailedAccessAttemptsBeforeLockout = 5;

            var dataProtectionProvider = options.DataProtectionProvider;
            if (dataProtectionProvider != null)
            {
                manager.UserTokenProvider =
                    new DataProtectorTokenProvider<User, int>(dataProtectionProvider.Create("ASP.NET Identity"));
            }
            return manager;
        }
    }

    // Configure the application sign-in manager which is used in this application.
    public class ApplicationSignInManager : SignInManager<User, int>
    {
        public ApplicationSignInManager(ApplicationUserManager userManager, IAuthenticationManager authenticationManager)
            : base(userManager, authenticationManager)
        {
        }

        //public override Task<ClaimsIdentity> CreateUserIdentityAsync(MyIdentityUser user)
        //{
        //    return user.GenerateUserIdentityAsync((ApplicationUserManager)UserManager);
        //}

        public static ApplicationSignInManager Create(IdentityFactoryOptions<ApplicationSignInManager> options, IOwinContext context)
        {
            return new ApplicationSignInManager(context.GetUserManager<ApplicationUserManager>(), context.Authentication);
        }

        public override async Task<SignInStatus> PasswordSignInAsync(string userName, string password, bool isPersistent, bool shouldLockout)
        {
            var user = await UserManager.FindByNameAsync(userName);
            if (user != null)
            {
                var result = UserStatusResult.UserNameInvalid;
                if (user.IsAdUser)
                {
                    // L'utente è di dominio.
                    var client = new AuthenticationClient();
                    result = await client.AuthenticateAsync(userName, password);

                    // recupera le informazioni da AD tramite la web api su Techorizon.Authorization
                    var aduser = await client.GetUserInformationAsync(userName);
                    if (aduser != null)
                    {
                        user.FirstName = aduser.FirstName;
                        user.LastName = aduser.LastName;
                        user.Email = aduser.Email;
                    }

                    if (result == UserStatusResult.Locked)
                    {
                        //imposto data lockout
                    }
                    else if (result == UserStatusResult.Disabled)
                    {
                        //imposto disabled
                    }
                    else if (result == UserStatusResult.ChangePasswordAtNextLogon || result == UserStatusResult.PasswordExpired || result == UserStatusResult.Ok)
                    {
                        //reset
                    }
                    await UserManager.UpdateAsync(user);
                }
                else
                {
                    // L'utente è solo Database
                    if (await this.UserManager.CheckPasswordAsync(user, password))
                    {
                        result = UserStatusResult.Ok;

                        //if (await this.UserManager.IsLockedOutAsync(user.Id))
                        //    result = UserStatusResult.Locked;
                    }
                    else
                    {
                        if (this.UserManager.UserLockoutEnabledByDefault)
                        {
                            // Record the failure which also may cause the user to be locked out
                            await UserManager.AccessFailedAsync(user.Id);

                            //if (await UserManager.IsLockedOutAsync(user.Id))
                            //{
                            //    result = UserStatusResult.Locked;
                            //}

                        }
                    }
                }


                switch (result)
                {
                    case UserStatusResult.Ok:
                        await SignInAsync(user, isPersistent: isPersistent, rememberBrowser: false);
                        return await Task.FromResult(SignInStatus.Success);
                    case UserStatusResult.ChangePasswordAtNextLogon:
                        return SignInStatus.RequiresVerification;
                    case UserStatusResult.Disabled:
                    case UserStatusResult.Locked:
                        return SignInStatus.LockedOut;
                    case UserStatusResult.WrongPassword:
                    case UserStatusResult.GenericError:
                    case UserStatusResult.PasswordExpired:
                    case UserStatusResult.UserNameInvalid:
                    case UserStatusResult.PasswordNull:
                    case UserStatusResult.InvalidDomainController:
                        return SignInStatus.Failure;
                }
            }

            return await Task.FromResult(SignInStatus.Failure);
            //    return await base.PasswordSignInAsync(userName, password, isPersistent, shouldLockout);
        }
    }
}
